<?php

namespace FiloBlu\ShipmentEmailTrack\Block\Adminhtml\System\Config\Form\Field;

class FieldsMapping extends \Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray {

    protected $_arrayRowsCache;

    /**
     * @var \Magento\Framework\Data\Form\Element\Factory
     */
    protected $_elementFactory;

    /**
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Framework\Data\Form\Element\Factory $elementFactory
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Data\Form\Element\Factory $elementFactory,
        array $data = []
    )
    {
        $this->_elementFactory  = $elementFactory;
        parent::__construct($context,$data);
    }

    protected function _construct()
    {
        $this->addColumn('carrier_code', ['label' => __('Carrier Code')]);
        $this->addColumn('carrier_title', ['label' => __('Carrier Title')]);
        $this->addColumn('base_url', ['label' => __('URL')]);
        $this->_addAfter = false;
        $this->_addButtonLabel = __('Add');
        parent::_construct();
    }

    public function getArrayRows() {
        if (null !== $this->_arrayRowsCache) {
            return $this->_arrayRowsCache;
        }
        $result = [];
        /** @var \Magento\Framework\Data\Form\Element\AbstractElement */
        $element = $this->getElement();

        if ($element->getValue() && !is_array($element->getValue())) {
            $element->setValue(unserialize($element->getValue()));
        }

        $columnSetted = array_keys($this->getColumns());
        if ($element->getValue() && is_array($element->getValue())) {
            foreach ($element->getValue() as $rowId => $row) {

                /*
                 * Fix for unserialize of the values
                 * Check if other columns was created
                 */
                if(count($row) < count($columnSetted))
                {
                    $rowColumn = array_keys($row);

                    // If new columns was created, I add the new column with empty value
                    foreach ($columnSetted as $col)
                    {
                        if(!in_array($col, $rowColumn))
                        {
                            $row[$col] = '';
                        }
                    }
                }

                $rowColumnValues = [];
                foreach ($row as $key => $value) {
                    $row[$key] = $value;
                    $rowColumnValues[$this->_getCellInputElementId($rowId, $key)] = $row[$key];
                }

                $row['_id'] = $rowId;
                $row['column_values'] = $rowColumnValues;
                $result[$rowId] = new \Magento\Framework\DataObject($row);
                $this->_prepareArrayRow($result[$rowId]);
            }
        }
        $this->_arrayRowsCache = $result;
        return $this->_arrayRowsCache;
    }
}
